unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ProblemDomain;

type
  TfrmMain = class(TForm)
    lblCustomerName: TLabel;
    edtCustomerName: TEdit;
    btnFind: TButton;
    lvwCustomers: TListView;
    btnDelete: TButton;
    btnEdit: TButton;
    btnNew: TButton;
    procedure btnNewClick(Sender: TObject);
    procedure btnEditClick(Sender: TObject);
    procedure lvwCustomersChange(Sender: TObject; Item: TListItem; Change: TItemChange);
    procedure btnDeleteClick(Sender: TObject);
  private
    procedure UpdateCustomerDetails (ListItem: TListItem; Customer: TCustomer);
    procedure GetSelectedCustomer (Customer: TCustomer);
  public
  end;

var
  frmMain: TfrmMain;

implementation

uses
  CustomerDetails;

const
  CRLF = Chr ($d) + Chr ($a);
  
{$R *.DFM}

procedure TfrmMain.UpdateCustomerDetails (ListItem: TListItem; Customer: TCustomer);
begin
  with Customer, ListItem do begin
    Caption := Name;
    SubItems.Clear;
    SubItems.Add (Postcode);
    SubItems.Add (StringReplace (Customer.Address.Text, CRLF, ', ', [rfReplaceAll]));
  end;
end;

procedure TfrmMain.GetSelectedCustomer (Customer: TCustomer);
begin
  // In the real world this routine would have some way of identifying the customer to be
  // loaded for the selected entry (maybe the object would be stored behind the list item
  // Data property, maybe there would be a property on the list item exposing the primary
  // key of the object or whatever). In this instance we'll just use the visible data to
  // update our object.
  Assert (lvwCustomers.Selected <> nil, 'No list item selected!');
  with lvwCustomers.Selected, Customer do begin
    Name := Caption;
    Postcode := SubItems[0];
    Address.Text := StringReplace (SubItems[1], ', ', CRLF, [rfReplaceAll]);
  end;
end;

// Events

procedure TfrmMain.btnNewClick (Sender: TObject);
var
  Customer: TCustomer;
begin
  Customer := TCustomer.Create;
  try
    if TfrmCustomerDetails.Edit (Customer) = mrOK then UpdateCustomerDetails (lvwCustomers.Items.Add, Customer);
  finally
    Customer.Free;
  end;
end;

procedure TfrmMain.btnEditClick (Sender: TObject);
var
  Customer: TCustomer;
begin
  if lvwCustomers.Selected = nil then Exit;
  Customer := TCustomer.Create;
  try
    GetSelectedCustomer (Customer);
    if TfrmCustomerDetails.Edit (Customer) = mrOK then UpdateCustomerDetails (lvwCustomers.Selected, Customer);
  finally
    Customer.Free;
  end;
end;

procedure TfrmMain.lvwCustomersChange(Sender: TObject; Item: TListItem; Change: TItemChange);
begin
  btnEdit.Enabled := (Item <> nil);
  btnDelete.Enabled := (Item <> nil);
end;

procedure TfrmMain.btnDeleteClick(Sender: TObject);
begin
  if lvwCustomers.Selected = nil then Exit;
  if MessageDlg ('Are you sure you want to delete ' + lvwCustomers.Selected.Caption + '?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then begin
    lvwCustomers.Selected.Delete;
  end;
end;

end.
